#!/usr/bin/perl
#
#
#  Copyright (c) 2004, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions and the following disclaimer in the documentation
#     and/or other materials provided with the distribution.
#   * Neither the name of SWITCH nor the names of its contributors may be
#     used to endorse or promote products derived from this software without
#     specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.
#
#  $Author: peter $
#
#  $Id: install.pl 25 2005-04-01 12:08:09Z peter $
#
#  $LastChangedRevision: 25 $

require 5.6.0;

use strict;

my $VERSION = '$Id: install.pl 25 2005-04-01 12:08:09Z peter $';
my $nfsen_version = "1.1rc3";

#
# These settings are configured in the config file
our %sources  = ();
our $USER     = undef;
our $WWWUSER  = undef;
our $WWWGROUP = undef;
our $BUFFLEN  = undef;
our $PREFIX   = undef;

my @ProfileTag = ( 
	"# \n",
	"# Profile live.\n",
	"# \n",
);

#
# user $user needs to exist and must be a member of $WWWGROUP
sub VerifyUser {
	my $user = shift;

	my $www_gid = getgrnam($NfConf::WWWGROUP);

	my ($login,$pass,$uid,$user_gid) = getpwnam($user);
	if ( !defined $login ) {
		die "user '$user' not found on this system\n";
	}

	# Verify if $user has gid of $WWWGROUP
	if ( $user_gid == $www_gid ) {
		return;
	}

	# Not yet found - check if $user is a member of $WWWGROUP
	my ($gid_name,$passwd,$group_gid,$group_members) = getgrgid($www_gid);
	if ( !defined $gid_name ) {
		die "Group '$user_gid' not found on this system\n";
	}
	# Check the members list
	foreach my $member ( split /\s+/, $group_members ) {
		if ( $member eq $user ) {
			# user found
			return;
		}
	}
	die "User '$user' not a member of group '$NfConf::WWWGROUP'\n";

} # End of VerifyUser

sub VerifyConfig {

	die "Missing PREFIX directory of nfdump tools!\n" unless defined $NfConf::PREFIX;
	die "'$PREFIX' not a directory!\n" unless -d  $NfConf::PREFIX;

	my $wwwgid = getgrnam($NfConf::WWWGROUP) || 
		die "WWW group '$NfConf::WWWGROUP' not found on this system\n";

	VerifyUser($NfConf::WWWUSER);
	VerifyUser($NfConf::USER);

	die "NFSEN does not want to run as root!\n" if $NfConf::USER eq 'root';

	my %_seen;
	foreach my $source ( keys %NfConf::sources ) {
		my $len = length($source);
		if ( $len < 1 || $len > 19 ) {
			die "Error Ident: '$source': Source identifiers must be between 1 and 19 chars long!\n";
		}
		if ( $source =~ /[^a-z0-9_]+/i ) {
			die "Error Ident: '$source': Source identifiers must contain only characters [a-zA-Z0-9_] !\n";
		}
		my $port = $NfConf::sources{$source}{'port'};
		if ( exists $_seen{$port} ) {
			die "Error Ident: '$source': Port $port already configured for source $_seen{$port}!\n";
		} else {
			$_seen{$port} = $source;
		}
	}

} # End of VerifyConfig

sub SetShellParams {

	my $ShellParamFile = "$NfConf::BASEDIR/etc/nfsen-shell-param";

	print "Update Shell vars file $ShellParamFile\n";
	open SHPARAM, ">$ShellParamFile" || die "Can't open Shell Parameter file: $!\n";
	print SHPARAM "#\n# Automatically generated by $0\n";
	print SHPARAM "BASEDIR=$NfConf::BASEDIR\n";
	print SHPARAM "PREFIX=$NfConf::PREFIX\n";
	print SHPARAM "USER='-u $NfConf::USER'\n";

	my $tmp = $NfConf::WWWGROUP  ? "'-g $NfConf::WWWGROUP'"   : "''";
	print SHPARAM "GROUP=$tmp\n";

	$tmp = $NfConf::BUFFLEN ? "'-B $NfConf::BUFFLEN'"   : "''";
	print SHPARAM "BUFFLEN=$tmp\n";

	my @src;
	foreach my $ident ( keys %NfConf::sources ) {
		push @src, "$ident:$NfConf::sources{$ident}{'port'}"
	}
	my $str = join ' ', @src;
	print SHPARAM "SOURCES='$str'\n";
	close SHPARAM;

	# note: s#^\\. : the second '\' is eaten up by the expansion qq{}
	print "Update nfcapd/NFSEN start/stop script: $NfConf::BASEDIR/bin/nfsen.rc\n";
	system qq{/usr/bin/perl -pi -e 's#^\\..+#. $ShellParamFile#' $NfConf::BASEDIR/bin/nfsen.rc};

} # End of SetShellParams

sub PatchLibraryPath {

	my $libpath = "$NfConf::BASEDIR/libexec";

	my @Scripts = ( 'nfsen', 'nfsen-run', 'testPlugin', 'GenGraph.pl' );
	my $trigger = '##########';

	foreach my $script ( @Scripts ) {
		print "Update Library path for: $script\n";
		open SCRIPT, "bin/$script" || die "Can't open Script $script for reading: $!\n";
		my @lines = <SCRIPT>;
		close SCRIPT;
		open SCRIPT, ">bin/$script" || die "Can't open Script $script for writing: $!\n";
		my $numtriggers = 0;
		my $found = 0;
		foreach my $line ( @lines ) {
			if ( $numtriggers == 0 ) {
				if ( $line =~ /$trigger/ ) {
					$numtriggers = 1;
				}
				print SCRIPT $line;
				next;
			}
			if ( $numtriggers == 1 ) {
				if ( $line =~ /^use lib/ ) {
					$line = "use lib \"$libpath\";\n";
					$found = 1;
				}
				if ( $line =~ /$trigger/ ) {
					$numtriggers = 2;
				}
				print SCRIPT $line;
				next;
			}
			if ( $numtriggers > 1 ) {
				print SCRIPT $line;
			}
		}
		close SCRIPT;
	}
	print "\n";

} # End of PatchLibraryPath

sub SetupHTML {

	print "Setup php and html files.\n";
	mkdir "$NfConf::HTMLDIR" unless -d "$NfConf::HTMLDIR";
	die "Could not create HTMl directory '$NfConf::HTMLDIR': $!\n"  unless -d $NfConf::HTMLDIR;

	system("/bin/cp html/* $NfConf::HTMLDIR");
	open CONF, ">$NfConf::HTMLDIR/conf.php" || die "Can't open conf.php for writing: $!\n";
	print CONF "<?\n";
	print CONF "/* This file was automatically created by nfsen-setup */\n\n";
	print CONF "\$BASEDIR = \"$NfConf::BASEDIR\";\n";
	print CONF "\$PREFIX  = \"$NfConf::PREFIX\";\n\n";
	print CONF "\$SOURCES = array ( ";
	my @sources =  keys %NfConf::sources;
	for ( my $i=0; $i<scalar @sources; $i++ ) {
		$sources[$i] = "'$sources[$i]'";
	}
	print CONF join ', ', @sources;
	print CONF " );\n";
	print CONF "\n?>\n";
	close CONF;
	print "\n";
	
} # End of SetupHTML

sub SetupEnv {
	

	# Make sure all required directories exist

	umask 0002;
	print "\nSetup diretories:\n";
	my @dirs = (
		"$NfConf::BASEDIR/var",
		"$NfConf::BASEDIR/var/run",
		"$NfConf::BASEDIR/var/filters",
		"$NfConf::BASEDIR/profiles",
		"$NfConf::BASEDIR/profiles/live",
	);

	my ($login,$pass,$uid,$gid) = getpwnam($NfConf::WWWUSER);
	if ( !defined $login ) {
			die "NFSEN user '$NfConf::WWWUSER' not found on this system\n";
	}

	if ( defined $NfConf::WWWGROUP ) {
		$gid  = getgrnam($NfConf::WWWGROUP) || 
			die "NFSEN group '$NfConf::WWWGROUP' not found on this system\n";
	}
	print "\nUse UID/GID $uid $gid\n";

	chown $uid, $gid, "$NfConf::BASEDIR" || die "Can't chown '$NfConf::BASEDIR': $!\n";

	foreach my $dir ( @dirs ) {
		if ( ! -d $dir ) {
			print "Creating: ";
			mkdir $dir || die "Can't mkdir '$dir': $!\n";
			chmod 0775, $dir || die "Can't chown '$dir': $!\n";
			chown $uid, $gid, $dir || die "Can't chown '$dir': $!\n";
		} else {
			print "Exists: ";
		}
		print "$dir\n";
	}

	print "\nProfile live: spool directories:\n";
	foreach my $ident ( keys %NfConf::sources ) {
		my $dir = "$NfConf::BASEDIR/profiles/live/$ident";
		if ( ! -d $dir ) {
			print "Creating: ";
			mkdir $dir || die "Can't mkdir '$dir': $!\n";
			chmod 0775, $dir || die "Can't chown '$dir': $!\n";
			chown $uid, $gid, $dir || die "Can't chown '$dir': $!\n";
		} else {
			print "Exists: ";
		}
		print "$ident\n";
	}

	NfSen::DropPriv($NfConf::WWWUSER);

	my $now = time();
	my $tstart = $now - ( $now % 300 );
	NfSenRRD::SetupRRD("$NfConf::BASEDIR/profiles/live", $tstart, 0);
	if ( $Log::ERROR ) {
		print "Error setup RRD DBs: $Log::ERROR\n";
	}

	my %profileinfo = NfSen::ReadProfile('live');
	if ( $profileinfo{'status'} eq 'empty' ) {
		print "Create profile info for profile 'live'\n";
		# empty - new profile
		$profileinfo{'_comment_'} 	= \@ProfileTag;
		$profileinfo{'name'}		= 'live';
		$profileinfo{'tstart'} 		= $tstart;
		$profileinfo{'tend'} 		= $tstart;
		$profileinfo{'updated'}		= $tstart - 300;
		$profileinfo{'expire'} 		= 0;
		$profileinfo{'maxsize'} 	= 0;
		$profileinfo{'size'} 		= 0;
		$profileinfo{'filter'}		= "";
		$profileinfo{'type'} 		= 0;
		$profileinfo{'locked'} 		= 0;
		$profileinfo{'status'} 		= 'rebuild';
	} else {
		print "Use existing profile info for profile 'live'\n";
	}
	$profileinfo{'sourcelist'}	= join ':', keys %NfConf::sources;

	NfSen::WriteProfile(\%profileinfo);
	
	print "\n";

} # End of SetupEnv

sub CopyFiles {

	print "Copy files ...\n";
	system("/bin/cp -r etc $NfConf::BASEDIR");
	system("/bin/cp -r bin $NfConf::BASEDIR");
	system("/bin/cp -r libexec $NfConf::BASEDIR");
	print "\n";

} # End of CopyFiles

########################
#
# Main starts here
#
########################

my $OldConfig = shift @ARGV;

unshift @INC, "libexec";

eval {
	require NfConf; 	import NfConf;
	require NfSen; 		import NfSen;
	require NfSenRRD; 	import NfSenRRD;
};

if ( $@ ) {
	print "Required nfsen modules not found\n";
	print "$@\n";
	exit;
}


if ( !NfSen::root_process() ) {
	die "nfsen setup wants to run as root\n";
}

if ( defined $OldConfig ) {
	if ( -f $OldConfig && ( "$OldConfig" ne "etc/nfsen.conf") ) {
		system("/bin/cp $OldConfig etc/nfsen.conf");
	} elsif ( ! -f $OldConfig ) {
		print "$OldConfig: No such file - ignored\n";
	}
}

if ( ! -f "etc/nfsen.conf" ) {
	die "Please configure etc/nfsen.conf first\n";
}
if ( !NfConf::LoadConfig(1) ) {
	print "$Log::ERROR\n";
	exit 1;
}

print "Setup NFSEN:\n";
print "Version: $nfsen_version: $VERSION\n\n";

VerifyConfig();
SetupHTML();
PatchLibraryPath();
CopyFiles();
SetShellParams();	
SetupEnv();

print "Setup done.\n";
exit 0;

